@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -34,6 +31,7 @@
 
 // IOService pulls in all the headers we need
 #include <IOKit/IOService.h>
+#include <IOKit/IOLib.h>
 #include "IOPlatformSensor.h"
 #include "IOPlatformControl.h"
 
@@ -42,7 +40,7 @@
 #endif
 
 // Uncomment for debug info
-// #define CTRLLOOP_DEBUG 1
+//#define CTRLLOOP_DEBUG 1
 
 #ifdef CTRLLOOP_DEBUG
 #define CTRLLOOP_DLOG(fmt, args...)  kprintf(fmt, ## args)
@@ -84,6 +82,7 @@ protected:
 enum {
 	kIOPCtrlLoopNotReady,
 	kIOPCtrlLoopFirstAdjustment,
+	kIOPCtrlLoopWillSleep,
 	kIOPCtrlLoopDidWake,
 	kIOPCtrlLoopAllRegistered
 };
@@ -174,6 +173,14 @@ public:
 	@param aControl An IOPlatformControl reference to the control to be removed. */
 	virtual bool removeControl( IOPlatformControl * aControl );
 
+	// control loops can use these functions to get notified when a sensor or control value
+	// is changed by another entity. This can be used to avoid duplicate polling/updating.
+	// the default implementation is simply to do nothing and return().
+	virtual void sensorCurrentValueWasSet( IOPlatformSensor * aSensor, SensorValue newValue );
+	virtual void controlCurrentValueWasSet( IOPlatformControl * aControl, ControlValue newValue );
+	virtual void controlTargetValueWasSet( IOPlatformControl * aControl, ControlValue newValue );
+
+	virtual void willSleep( void );
 	virtual void didWake( void );
 };
 
