@@ -30,7 +30,10 @@
 #include <sys/cdefs.h>
 
 __BEGIN_DECLS
+#if defined( __ppc__ )
 #include <ppc/proc_reg.h>
+#endif
+#include <machine/machine_routines.h>
 __END_DECLS
 
 #include <IOKit/IODeviceTreeSupport.h>
@@ -43,6 +46,10 @@ __END_DECLS
 
 #define kMacRISC_GPIO_DIRECTION_BIT	2
 
+#ifndef kIOHibernateStateKey
+#define kIOHibernateStateKey	"IOHibernateState"
+#endif
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #define super IOCPU
@@ -66,6 +73,11 @@ static const cpu_timebase_params_t pulsar =
 	{ kU3IIC, /* port */ 0x00, /* addr */ 0xD4, /* subaddr */ 0x2E,
 	  /* mask */ 0x77, /* enable */ 0x22, /* disable */ 0x11 };
 
+/* RackMac3,1 and PowerMac7,3 - pulsar clock chip @ IIC_B 0xD2 */
+static const cpu_timebase_params_t pulsarD2 =
+        { kU3IIC, /* port */ 0x00, /* addr */ 0xD2, /* subaddr */ 0x2E,
+          /* mask */ 0x77, /* enable */ 0x22, /* disable */ 0x11 };
+
 /*
  * These global variables are used for I2C communication in enableCPUTimeBase
  * The process is initiated in startCPU for CPU 1 but is completed in
@@ -81,9 +93,13 @@ static const cpu_timebase_params_t pulsar =
  *
  * This will have to change if we have more than two CPUs.
  */
-static bool gI2CTransactionComplete;
-static IOService *gI2CDriver;
-static const cpu_timebase_params_t * gTimeBaseParams;
+static bool								gI2CTransactionComplete;
+static IOService						*gI2CDriver,
+										*gTBDriver;
+static const OSSymbol					*gTBFunctionNameSym;
+static const OSSymbol					*gTBReadFunctionNameSym;		// XXX Temp!!!
+static const cpu_timebase_params_t		*gTimeBaseParams;
+static UInt32 							*gPHibernateState;
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -92,7 +108,7 @@ static IOCPUInterruptController *gCPUIC;
 bool MacRISC4CPU::start(IOService *provider)
 {
     kern_return_t        result;
-    IORegistryEntry      *cpusRegEntry, *mpicRegEntry;
+    IORegistryEntry      *cpusRegEntry, *cpu0RegEntry, *mpicRegEntry;
     OSIterator           *cpusIterator;
     OSData               *tmpData;
     IOService            *service;
@@ -102,9 +118,9 @@ bool MacRISC4CPU::start(IOService *provider)
     OSDictionary         *matchDict;
     char                 mpicICName[48];
     UInt32               maxCPUs, physCPU, mpicPHandle;
-    IOService            *i2cresources;
+    IOService            *tbResources;
     ml_processor_info_t  processor_info;
-		
+				
     // callPlatformFunction symbols
     mpic_getProvider = OSSymbol::withCString("mpic_getProvider");
     mpic_getIPIVector= OSSymbol::withCString("mpic_getIPIVector");
@@ -131,6 +147,11 @@ bool MacRISC4CPU::start(IOService *provider)
   
     if (!super::start(provider)) return false;
 
+    // Find out if this is the boot CPU.
+    tmpData = OSDynamicCast(OSData, provider->getProperty("state"));
+    if (tmpData == 0) return false;
+    bootCPU = (strcmp((char *)tmpData->getBytesNoCopy(), "running") == 0);
+
     // Count the CPUs.
     numCPUs = 0;
     cpusRegEntry = fromPath("/cpus", gIODTPlane);
@@ -139,6 +160,12 @@ bool MacRISC4CPU::start(IOService *provider)
     while (cpusIterator->getNextObject()) numCPUs++;
     cpusIterator->release();
   
+	// [3775206] - The bootCPU driver inits globals for all instances (like gCPUIC) so if we're not the
+	// boot CPU driver we wait here for that driver to finish its initialization
+	if ((numCPUs > 1) && !bootCPU)
+		// Wait for bootCPU driver to say it's up and running
+		(void) waitForService (resourceMatching ("BootCPU"));
+
     // Limit the number of CPUs by the cpu=# boot arg.
     if (PE_parse_boot_arg("cpus", &maxCPUs))
     {
@@ -146,55 +173,92 @@ bool MacRISC4CPU::start(IOService *provider)
     }
 
 	// If we're MP, fetch the timebase parameters.  If we can't sync timebase, limit the number of CPUs to 1.
-	if (numCPUs > 1 && !gTimeBaseParams)
-	{
-		IORegistryEntry * clockchip;
+	if (numCPUs > 1 && (!(gTimeBaseParams || gTBDriver))) {
+		// First try for standard platform function way.  This usually just requires toggling a GPIO so it
+		// is very easy and we get a reference to the GPIO driver to do it for us
+		if (cpusRegEntry->getProperty ("platform-cpu-timebase")) {
+			// OK a platform function handler exists so go find it
+			OSData *pHandle;
+			char	functionName[64];
+			
+			if ((pHandle = OSDynamicCast( OSData, cpusRegEntry->getProperty("AAPL,phandle") )) != NULL) {
+				// Build the call plaform function name by appending our phandle
+				sprintf(functionName, "%s-%08lx", "platform-cpu-timebase", *((UInt32 *)pHandle->getBytesNoCopy()));
+				//MYLOG ("4CPU: got tb function '%s'\n", functionName);
+				gTBFunctionNameSym = OSSymbol::withCString(functionName);
 
-		// PowerMac7,2 (CY28508) and PowerMac7,3 (Pulsar)
-		if ((strcmp(macRISC4PE->provider_name, "PowerMac7,2") == 0) || (strcmp(macRISC4PE->provider_name, "PowerMac7,3") == 0))
-		{
-			// look for cypress at slave address 0xd2
-			if ((clockchip = fromPath("/u3/i2c/i2c-hwclock@d2", gIODTPlane, 0, 0, 0)) != NULL)
-			{
-				clockchip->release();
-				gTimeBaseParams = &cypress;
+				// XXX - This is temporary for Waveland bringup
+				if (cpusRegEntry->getProperty ("platform-read-cpu-timebase")) {
+					sprintf(functionName, "%s-%08lx", "platform-read-cpu-timebase", 
+						*((UInt32 *)pHandle->getBytesNoCopy()));
+					//MYLOG ("4CPU: got tb read function '%s'\n", functionName);
+					gTBReadFunctionNameSym = OSSymbol::withCString(functionName);
+				}
 			}
+		} else {
+			/*
+			 * Bummer, we don't have an easy way.  Instead we need to do an I2C transaction to the clock driver
+			 * This turns out to be tricky because the disable code needs to run from an interrupt context
+			 * See startCPU() for more explanation
+			 */
+			IORegistryEntry * clockchip;
+	
+			// PowerMac7,2 (CY28508) and PowerMac7,3 (Pulsar)
+			if ((strcmp(macRISC4PE->provider_name, "PowerMac7,2") == 0) || (strcmp(macRISC4PE->provider_name, "PowerMac7,3") == 0)) {
+				// look for cypress/pulsar at slave address 0xd2
+				if ((clockchip = fromPath("/u3/i2c/i2c-hwclock@d2", gIODTPlane, 0, 0, 0)) != NULL) {
+					OSString *pulsarCompat, *cypressCompat;
+					
+					pulsarCompat = OSString::withCString ("pulsar-legacy-slewing");
+					cypressCompat = NULL;
+					if (IODTCompareNubName(clockchip, pulsarCompat, NULL)) {
+						gTimeBaseParams = &pulsarD2;
+						IOLog ("MacRISC4CPU::start - found 'pulsar-legacy-slewing'\n");
+					} else {
+						cypressCompat = OSString::withCString ("cy28508");
+						if (IODTCompareNubName(clockchip, cypressCompat, NULL)) {
+							gTimeBaseParams = &cypress;
+							IOLog ("MacRISC4CPU::start - found 'cy28508'\n");
+						}
+						else IOLog ("MacRISC4CPU::start - no match for 'i2c-hwclock@d2'\n");
+					}
 
-			// look for pulsar at slave address 0xd4
-			else if ((clockchip = fromPath("/u3/i2c/i2c-hwclock@d4", gIODTPlane, 0, 0, 0)) != NULL)
-			{
-				clockchip->release();
-				gTimeBaseParams = &pulsar;
+					pulsarCompat->release();
+					if (cypressCompat) cypressCompat->release();
+				} else if ((clockchip = fromPath("/u3/i2c/i2c-hwclock@d4", gIODTPlane, 0, 0, 0)) != NULL) {
+					// look for pulsar at slave address 0xd4
+					gTimeBaseParams = &pulsar;
+				}
+				if (clockchip) clockchip->release();
 			}
-		}
-
-		// RackMac3,1 (Pulsar)
-		else if (strcmp(macRISC4PE->provider_name, "RackMac3,1") == 0)
-		{
-			// look for pulsar at slave address 0xd4
-			if ((clockchip = fromPath("/u3/i2c/i2c-hwclock@d4", gIODTPlane, 0, 0, 0)) != NULL)
+	
+			// RackMac3,1 (Pulsar)
+			else if (strcmp(macRISC4PE->provider_name, "RackMac3,1") == 0)
 			{
-				clockchip->release();
-				gTimeBaseParams = &pulsar;
+				// look for pulsar at slave address 0xd4
+				if ((clockchip = fromPath("/u3/i2c/i2c-hwclock@d4", gIODTPlane, 0, 0, 0)) != NULL)
+				{
+					clockchip->release();
+					gTimeBaseParams = &pulsar;
+				}
 			}
 		}
 
 		// if we haven't figured out how to sync timebase, then limit the number of CPUs to 1
-		if (!gTimeBaseParams)
-		{
-			IOLog("WARNING: don't know how to sync MP timebase, limiting to one CPU\n");
-			kprintf("WARNING: don't know how to sync MP timebase, limiting to one CPU\n");
-			numCPUs = 1;
-		}
+		if (!(gTimeBaseParams || gTBFunctionNameSym)) {
+				IOLog("WARNING: don't know how to sync MP timebase, limiting to one CPU\n");
+				kprintf("WARNING: don't know how to sync MP timebase, limiting to one CPU\n");
+				numCPUs = 1;
+			}
 	}
 	
 	doSleep = false;
   
     // Get the "flush-on-lock" property from the first cpu node.  "flush-on-lock" mean do not "nap" the processor.
     flushOnLock = false;
-    cpusRegEntry = fromPath("/cpus/@0", gIODTPlane);
-    if (cpusRegEntry == 0) return false;
-    if (cpusRegEntry->getProperty("flush-on-lock") != 0) flushOnLock = true;
+    cpu0RegEntry = fromPath("/cpus/@0", gIODTPlane);
+    if (cpu0RegEntry == 0) return false;
+    if (cpu0RegEntry->getProperty("flush-on-lock") != 0) flushOnLock = true;
   
     // Set flushOnLock when numCPUs is not one.
     //if (numCPUs != 1) flushOnLock = true;
@@ -205,21 +269,17 @@ bool MacRISC4CPU::start(IOService *provider)
     physCPU = *(long *)tmpData->getBytesNoCopy();
     setCPUNumber(physCPU);
 
-    // Get the gpio offset for soft reset from the "soft-reset" property.
-    tmpData = OSDynamicCast(OSData, provider->getProperty("soft-reset"));
-    if (tmpData == 0) 
-    {
-		IOLog ("MacRISC4CPU::start - cpu(%ld) - no soft-reset property\n", physCPU);
-		return false;
-    } else
-        soft_reset_offset = *(long *)tmpData->getBytesNoCopy();
+	if (numCPUs > 1) {
+		// Get the gpio offset for soft reset from the "soft-reset" property.
+		tmpData = OSDynamicCast(OSData, provider->getProperty("soft-reset"));
+		if (tmpData == 0) 
+		{
+			IOLog ("MacRISC4CPU::start - cpu(%ld) - no soft-reset property\n", physCPU);
+			return false;
+		} else
+			soft_reset_offset = *(long *)tmpData->getBytesNoCopy();
+	}
    
-    // Find out if this is the boot CPU.
-    bootCPU = false;
-    tmpData = OSDynamicCast(OSData, provider->getProperty("state"));
-    if (tmpData == 0) return false;
-    if (!strcmp((char *)tmpData->getBytesNoCopy(), "running")) bootCPU = true;
-  
     if (bootCPU)
     {
         gCPUIC = new IOCPUInterruptController;
@@ -234,8 +294,10 @@ bool MacRISC4CPU::start(IOService *provider)
     tmpData = OSDynamicCast(OSData, provider->getProperty("l2cr"));
     if (tmpData != 0)
         l2crValue = *(long *)tmpData->getBytesNoCopy() & 0x7FFFFFFF;
+#if defined( __ppc__ )
     else
         l2crValue = mfl2cr() & 0x7FFFFFFF;
+#endif
   
 	kprintf ("MacRISC4CPU::start - waiting for KeyLargo\n");
     // Wait for KeyLargo to show up.
@@ -283,31 +345,6 @@ bool MacRISC4CPU::start(IOService *provider)
         provider->joinPMtree(this);
     }
 
-#if 0		// XXX - incomplete
-	if (macRISC4PE->ioPPluginNub) {
-		// Find the platform monitor, if present
-		service = waitForService(resourceMatching("IOPlatformPlugin"));
-		ioPPlugin = OSDynamicCast (IOService, service->getProperty("IOPlatformPlugin"));
-		if (!ioPPlugin) 
-			return false;
-		
-		ioPPluginDict = OSDictionary::withCapacity(2);
-		if (!ioPPluginDict) {
-			ioPPlugin = NULL;
-		} else {
-			ioPPluginDict->setObject (kIOPPluginTypeKey, OSSymbol::withCString (kIOPPluginTypeCPUCon));
-			ioPPluginDict->setObject (kIOPPluginCPUIDKey, OSNumber::withNumber ((long long)getCPUNumber(), 32));
-
-			if (messageClient (kIOPPluginMessageRegister, ioPPlugin, (void *)ioPPluginDict) != kIOReturnSuccess) {
-				IOLog ("MacRISC4CPU::start - failed to register cpu with IOPlatformPlugin\n");
-				ioPPluginDict->release();
-				ioPPlugin = NULL;
-				return false;
-			}
-		}
-	}
-#endif
-
     registerService();
 
 	if (!(uniN = waitForService(serviceMatching("AppleU3")))) return false;
@@ -319,25 +356,51 @@ bool MacRISC4CPU::start(IOService *provider)
 	if (numCPUs == 1) 
 		uniN->callPlatformFunction (u3APIPhyDisableProcessor1, false, (void *)0, (void *)0, (void *)0, (void *)0);
 
-	// We have to get a pointer to the I2C driver now -- when we're syncing timebase, we'll be at
-	// interrupt context
-	if (numCPUs > 1 && !gI2CDriver)
-	{
-		//kprintf("MacRISC4CPU(%ld) looking for %s\n", getCPUNumber(), gTimeBaseParams->i2c_iface);
-		i2cresources = waitForService (resourceMatching ( gTimeBaseParams->i2c_iface ));
-		//kprintf("MacRISC4CPU(%ld) done looking\n", getCPUNumber());
-
-		if (i2cresources)
-		{
-			gI2CDriver = OSDynamicCast (IOService, i2cresources->getProperty ( gTimeBaseParams->i2c_iface ));
-			if (!gI2CDriver)
-			{
-				kprintf ("MacRISC4CPU::start(%ld) - failed i2cDriver\n", getCPUNumber());
-				return false;
+	// We have to get a pointer to the timebase driver now -- when we're syncing timebase, we'll be at
+	// interrupt context and can't go looking for drivers
+	if (numCPUs > 1) {
+		if (gTimeBaseParams && !gI2CDriver) {
+			//kprintf("MacRISC4CPU(%ld) looking for %s\n", getCPUNumber(), gTimeBaseParams->i2c_iface);
+			tbResources = waitForService (resourceMatching ( gTimeBaseParams->i2c_iface ));
+			//kprintf("MacRISC4CPU(%ld) done looking\n", getCPUNumber());
+	
+			if (tbResources) {
+				gI2CDriver = OSDynamicCast (IOService, tbResources->getProperty ( gTimeBaseParams->i2c_iface ));
+				if (!gI2CDriver)
+				{
+					kprintf ("MacRISC4CPU::start(%ld) - failed i2cDriver\n", getCPUNumber());
+					return false;
+				}
 			}
+		} else if (gTBFunctionNameSym && !gTBDriver) {
+#if 0
+			mach_timespec_t waitTimeout;
+			
+			waitTimeout.tv_sec = 15;
+			waitTimeout.tv_nsec = 0;
+	
+			// MYLOG ("4CPU::startCPU - waiting for TBDriver with 15 second timeout\n");
+			tbResources = waitForService(resourceMatching(gTBFunctionNameSym), &waitTimeout);
+#else
+			// MYLOG ("4CPU::startCPU - waiting for TBDriver\n");
+			tbResources = waitForService(resourceMatching(gTBFunctionNameSym));
+#endif
+			if (tbResources) 
+				gTBDriver = OSDynamicCast (IOService, tbResources->getProperty(gTBFunctionNameSym));
+			// else
+				// MYLOG ("4CPU::startCPU - timeout on TBDriver\n");
+			
+			// if (gTBDriver) MYLOG ("4CPU::startCPU - got TB service\n");
 		}
 	}
 
+	// necessary bootCPU initialization is done, so release other CPU drivers to do their thing
+	// other drivers need to be unblocked *before* we call processor_start otherwise we deadlock
+	if (bootCPU) {
+		IOLog ("MacRISC4CPU: publishing BootCPU\n");
+		publishResource ("BootCPU", this);
+	}
+
     if (physCPU < numCPUs)
     {
         processor_info.cpu_id           = (cpu_id_t)this;
@@ -347,11 +410,12 @@ bool MacRISC4CPU::start(IOService *provider)
         processor_info.supports_nap     = !flushOnLock;
         processor_info.time_base_enable = MacRISC4CPU::sEnableCPUTimeBase;
 
+#if defined( __ppc__ )
 		OSData*							powerTuneData;
 
 		// power_mode_0 and power_mode_1 are defined in a newer version of ppc/machine_routines.h.
 
-		if ( ( powerTuneData = OSDynamicCast( OSData, cpusRegEntry->getProperty( "power-mode-data" ) ) ) != NULL )
+		if ( ( powerTuneData = OSDynamicCast( OSData, cpu0RegEntry->getProperty( "power-mode-data" ) ) ) != NULL )
 		{
 			uint32_t*					powerTuneEntry;
 
@@ -365,11 +429,15 @@ bool MacRISC4CPU::start(IOService *provider)
 				processor_info.power_mode_1 = powerTuneEntry[ 1 ];
 				}
 		}
+#endif
 
 		// Register this CPU with mach.
+
+#if defined( __ppc__ )
 		kprintf("MacRISC4CPU::start(%ld) - registering with mach\n", getCPUNumber());
 		result = ml_processor_register(&processor_info, &machProcessor,	&ipi_handler);
 		if (result == KERN_FAILURE) return false;
+#endif
 	
 		processor_start(machProcessor);
 	}
@@ -390,40 +458,13 @@ bool MacRISC4CPU::start(IOService *provider)
 	// Call UniN to prepare for that, if necessary
 	uniN->callPlatformFunction ("setupUATAforSleep", false, (void *)0, (void *)0, (void *)0, (void *)0);
 
+	if (cpusRegEntry) cpusRegEntry->release();
+	if (cpu0RegEntry) cpu0RegEntry->release();
+	
 	kprintf ("MacRISC4CPU::start(%ld) - done\n", getCPUNumber());
     return true;
 }
 
-void MacRISC4CPU::performPMUSpeedChange (UInt32 newLevel)
-{
-    bool				tempRememberNap;
-
-	// Note the current processor speed so quiesceCPU knows what to do
-	currentProcessorSpeed = newLevel;
-	
-	// Disable nap to prevent PMU doing reset too soon.
-	tempRememberNap = ml_enable_nap(getCPUNumber(), false);
-	
-	kprintf ("performPMUSpeedChange - initiating speed change to level %ld, nap %d\n", newLevel, tempRememberNap);
-	IOLog ("performPMUSpeedChange - initiating speed change to level %ld, nap %d\n", newLevel, tempRememberNap);
-	
-	// Set flags for processor speed change.
-	processorSpeedChange = true;
-	
-	// Ask PM to do the processor speed change.
-	pmRootDomain->receivePowerNotification(kIOPMProcessorSpeedChange);
-	
-	// Set flags for system sleep.
-	processorSpeedChange = false;
-	
-	kprintf ("performPMUSpeedChange - completing speed change to level %ld, nap %d\n", newLevel, tempRememberNap);
-	IOLog ("performPMUSpeedChange - completing speed change to level %ld, nap %d\n", newLevel, tempRememberNap);
-	// Enable nap as needed.
-	ml_enable_nap(getCPUNumber(), tempRememberNap);
-	
-	return;
-}
-
 /* initCPU - not called at interrupt context but must not block */
 void MacRISC4CPU::initCPU(bool boot)
 {
@@ -451,12 +492,12 @@ void MacRISC4CPU::initCPU(bool boot)
 					// Got the driver - send the message
 					pciDriver->setDevicePowerState (NULL, 3);
 
-
 			keyLargo->callPlatformFunction(keyLargo_restoreRegisterState, false, 0, 0, 0, 0);
 	
-			// Disables the interrupts for this CPU.
+			// Enables the interrupts for this CPU.
 			if (macRISC4PE->getMachineType() == kMacRISC4TypePowerMac) {
-				kprintf("MacRISC4CPU::initCPU %d -> mpic->setUpForSleep on", getCPUNumber());
+				haveSleptMPIC = false;
+				kprintf("MacRISC4CPU::initCPU %d -> mpic->setUpForSleep off", getCPUNumber());
 				mpic->callPlatformFunction(mpic_setUpForSleep, false, (void *)false, (void *)getCPUNumber(), 0, 0);
 			}
 		}
@@ -469,7 +510,10 @@ void MacRISC4CPU::initCPU(bool boot)
         keyLargo->callPlatformFunction(keyLargo_syncTimeBase, false, 0, 0, 0, 0);
   
     if (boot) {
-        gCPUIC->enableCPUInterrupt(this);
+		if (gCPUIC)
+        	gCPUIC->enableCPUInterrupt(this);
+		else
+			panic ("MacRISC4CPU: gCPUIC uninitialized for CPU %d\n", getCPUNumber());
     
         // Register and enable IPIs.
         cpuNub->registerInterrupt(0, this, MacRISC4CPU::sIPIHandler, 0);
@@ -490,12 +534,17 @@ void MacRISC4CPU::quiesceCPU(void)
 {
     if (bootCPU)
     {
-        // Set the sleeping state for HWInit.
-		// Tell U3 to enter normal mode.
-		// For U3, this has to be done before telling the PMU to start going to sleep
-		uniN->callPlatformFunction (UniNSetPowerState, false, (void *)(kUniNSleep),
+        // Have U3 save state
+		uniN->callPlatformFunction (UniNSetPowerState, false, (void *)(kUniNSave),
 			(void *)0, (void *)0, (void *)0);
 
+		if (!gPHibernateState || !*gPHibernateState) {
+			// Tell U3 to enter sleep mode if not hibernating
+			// For U3, this has to be done before telling the PMU to start going to sleep
+			uniN->callPlatformFunction (UniNSetPowerState, false, (void *)(kUniNSleep),
+				(void *)0, (void *)0, (void *)0);
+        }
+
         if (processorSpeedChange) {
             // Send PMU command to speed the system
             pmu->callPlatformFunction("setSpeedNow", false, (void *)currentProcessorSpeed, 0, 0, 0);
@@ -506,25 +555,33 @@ void MacRISC4CPU::quiesceCPU(void)
 			 * After the command to PMU is sent we only have 100 milliseconds to quiesce
 			 * the cpu, so unnecessary delays, like kprints must be avoided.
 			 */
-			pmu->callPlatformFunction("sleepNow", false, 0, 0, 0, 0);
+			if (!gPHibernateState || !*gPHibernateState)
+				pmu->callPlatformFunction("sleepNow", false, 0, 0, 0, 0);
 	
-			// Enables the interrupts for this CPU.
-			if (macRISC4PE->getMachineType() == kMacRISC4TypePowerMac) 
+			// Disables the interrupts for this CPU.
+			if (!haveSleptMPIC && (macRISC4PE->getMachineType() == kMacRISC4TypePowerMac))
+			{
+				haveSleptMPIC = true;
 				mpic->callPlatformFunction(mpic_setUpForSleep, false, (void *)true, (void *)getCPUNumber(), 0, 0);
+			}
 	
 			// Save KeyLargo's register state.
 			keyLargo->callPlatformFunction(keyLargo_saveRegisterState, false, 0, 0, 0, 0);
 	
 			// Turn Off all KeyLargo I/O.
-			keyLargo->callPlatformFunction(keyLargo_turnOffIO, false, (void *)false, 0, 0, 0);
+			if (!gPHibernateState || !*gPHibernateState) {
+				keyLargo->callPlatformFunction(keyLargo_turnOffIO, false, (void *)false, 0, 0, 0);
+        	}
         }
         
         // Set the wake vector to point to the reset vector
         ml_phys_write(0x0080, 0x100);
 	}
 
+#if defined( __ppc__ )
     ml_ppc_sleep();
-	
+#endif
+
 	return;
 }
 
@@ -537,26 +594,32 @@ kern_return_t MacRISC4CPU::startCPU(vm_offset_t /*start_paddr*/, vm_offset_t /*a
 	//kprintf("MacRISC4CPU::startCPU(%ld) - entered\n", getCPUNumber());
 
 	// check for timebase sync info
-	if (!gTimeBaseParams)
+	if (!(gTimeBaseParams || gTBDriver))
 	{
-		IOLog("MacRISC4CPU::startCPU(%ld) - timebase sync params unknown\n", getCPUNumber() );
+		IOLog("MacRISC4CPU::startCPU(%ld) - cannot manage timebase synchronization\n", getCPUNumber() );
 		return KERN_FAILURE;
 	}
 
 	/*
 	 * On MP machines, once we enable the second CPU the kernel will want to sync the timebases
 	 * between the CPUs and will call enableCPUTimeBase to disable then enable the timebase around
-	 * the sync operation.  enableCPUTimeBase is called on an interrupt context and so it cannot block
+	 * the sync operation.  enableCPUTimeBase is called on an interrupt context and so it cannot block.
+	 * First generation G5 machines required an I2C transaction to the clock chip to disable the timebase
 	 * To ensure exclusive access to the I2C bus by enableCPUTimeBase, we open the I2C bus here, where
 	 * it is safe to block, then wait for the I2C transaction to complete after which we can close out
 	 * the bus.
+	 *
+	 * Later machines (those for which gTBDriver is non-NULL) use a GPIO instead and in enableCPUTimeBase
+	 * we can make a non-blocking call into the GPIO driver so this nonsense isn't necessary.
 	 */
-	gI2CTransactionComplete = false;
-	if (gI2CDriver && (kIOReturnSuccess == gI2CDriver->callPlatformFunction (i2c_openI2CBus, false,
-			(void *) (UInt32)gTimeBaseParams->i2c_port, (void *) 0, (void *) 0, (void *) 0))) {
-		kprintf ("MacRISC4CPU::startCPU(%ld) i2c bus opened\n", getCPUNumber());
-		// pre-set for combined mode
-		gI2CDriver->callPlatformFunction (i2c_setCombinedMode, false, (void *) 0, (void *) 0, (void *) 0, (void *) 0);	//CY28510 does reads in combined mode
+	if (gTimeBaseParams) {
+		gI2CTransactionComplete = false;
+		if (gI2CDriver && (kIOReturnSuccess == gI2CDriver->callPlatformFunction (i2c_openI2CBus, false,
+				(void *) (UInt32)gTimeBaseParams->i2c_port, (void *) 0, (void *) 0, (void *) 0))) {
+			kprintf ("MacRISC4CPU::startCPU(%ld) i2c bus opened\n", getCPUNumber());
+			// pre-set for combined mode
+			gI2CDriver->callPlatformFunction (i2c_setCombinedMode, false, (void *) 0, (void *) 0, (void *) 0, (void *) 0);	//CY28510 does reads in combined mode
+		}
 	}
 	
     // Strobe the reset line for this CPU.
@@ -584,11 +647,13 @@ kern_return_t MacRISC4CPU::startCPU(vm_offset_t /*start_paddr*/, vm_offset_t /*a
                                     (void *)&gpioOffset, (void *)strobe_value, 0, 0);
 
 	// Now wait for I2C transaction, handled by enableCPUTimeBase, to complete
-	while (!gI2CTransactionComplete) IOSleep (10);			// Spin until done
+	if (gTimeBaseParams) {
+		while (!gI2CTransactionComplete) IOSleep (10);			// Spin until done
 
-	gI2CDriver->callPlatformFunction (i2c_closeI2CBus, false, (void *) 0, (void *) 0, (void *) 0, (void *) 0);	//CY28510 does reads in combined mode
+		gI2CDriver->callPlatformFunction (i2c_closeI2CBus, false, (void *) 0, (void *) 0, (void *) 0, (void *) 0);	//CY28510 does reads in combined mode
 
-	kprintf ("MacRISC4CPU::startCPU(%ld) i2c bus closed\n", getCPUNumber());
+		kprintf ("MacRISC4CPU::startCPU(%ld) i2c bus closed\n", getCPUNumber());
+	}
 	
     return KERN_SUCCESS;
 }
@@ -606,6 +671,12 @@ void MacRISC4CPU::haltCPU(void)
   
     if (bootCPU)
     {
+		if (!gPHibernateState) {
+			OSData * data = OSDynamicCast(OSData, getPMRootDomain()->getProperty(kIOHibernateStateKey));
+			if (data)
+				gPHibernateState = (UInt32 *) data->getBytesNoCopy();
+		}
+        
 		uniN->callPlatformFunction (UniNPrepareForSleep, false, 
 			(void *)0, (void *)0, (void *)0, (void *)0);
 		// Notify our pci children to save their state
@@ -661,51 +732,98 @@ void MacRISC4CPU::signalCPU(IOCPU *target)
 }
 
 void MacRISC4CPU::sEnableCPUTimeBase( cpu_id_t self, boolean_t enable )
-	{
+{
 	MacRISC4CPU*			pe = ( MacRISC4CPU * ) self;
 
 	pe->enableCPUTimeBase( enable );
-	}
+}
 
+/*
+ * Read the CPU timebase registers.  This is not normally necessary so it's compiled
+ * out.  But I'm keeping it around because it's handy during development to make
+ * sure we are enabling/disabling the timebase correctly
+ */
+#if 0
+static void getCPUTimebase (UInt32 *tbuRet, UInt32 *tblRet)
+{
+	UInt32 tbu, tbu2, tbl;
+		
+	do {
+		asm volatile("  mftbu %0" : "=r" (tbu));
+		asm volatile("  mftb %0" : "=r" (tbl));
+		asm volatile("  mftbu %0" : "=r" (tbu2));
+	} while (tbu != tbu2);
+	
+	*tbuRet = tbu;
+	*tblRet = tbl;
+	
+	return;
+}
+#endif
 
 /* enableCPUTimeBase - IS called at interrupt context */
 void MacRISC4CPU::enableCPUTimeBase(bool enable)
 {
-	UInt8 sevenBitAddr, buf, tmp;
-
-	//kprintf("MacRISC4CPU::enableCPUTimeBase(%s)\n", enable ? "enable" : "disable");
-
-/*
-	kprintf ("MacRISC4CPU::enableCPUTimeBase ( %ld, %s ) port=%02lX addr=%02lX subaddr=%02lX mask=%02lX en=%02lX dis=%02lX\n",
-			getCPUNumber(),
-			enable ? "enable" : "disable",
-			gTimeBaseParams->i2c_port,
-			gTimeBaseParams->i2c_addr,
-			gTimeBaseParams->i2c_subaddr,
-			gTimeBaseParams->mask,
-			gTimeBaseParams->enable_value,
-			gTimeBaseParams->disable_value );
-*/
-
-	// read the byte register -- requires 7 bit slave address
-	sevenBitAddr = gTimeBaseParams->i2c_addr >> 1;
-	if (kIOReturnSuccess == gI2CDriver->callPlatformFunction (i2c_readI2CBus, false, (void *)(UInt32)sevenBitAddr, (void *) (UInt32)gTimeBaseParams->i2c_subaddr, (void *)(UInt32)&buf, (void *)1)) {
-		// apply mask and value
-		tmp = enable ? gTimeBaseParams->enable_value : gTimeBaseParams->disable_value; 
-		buf = (buf & ~gTimeBaseParams->mask) | (tmp & gTimeBaseParams->mask);
-		gI2CDriver->callPlatformFunction (i2c_writeI2CBus, false, (void *)(UInt32)sevenBitAddr,(void *) (UInt32)gTimeBaseParams->i2c_subaddr, (void *)(UInt32)& buf, (void *)1);
+	if (gTBDriver) {
+		UInt32		gpioValue;
+		UInt32 		count = 0;
+		
+		// XXX - temp for Waveland bringup - for Waveland we always drive the GPIO low
+		// SMU will set it high when it is done which is how we know it is complete - see
+		// the poll loop below.
+		// For normal systems, the GPIO itself drives the timebase signal so we just have
+		// to set it high or low to control it.
+		if (gTBReadFunctionNameSym) 
+			gpioValue = 0;
+		else
+			gpioValue = enable ? 1 : 0;
+		
+		// GPIO driver can handle it
+		gTBDriver->callPlatformFunction (gTBFunctionNameSym, false, (void *)gpioValue, 0, 0, 0);
+		
+		// XXX - temp for Waveland bringup
+		if (gTBReadFunctionNameSym) {
+			UInt32 gpioState;
+				
+			do {
+								
+				count++;
+				// Wait for SMU to restore GPIO state
+				gTBDriver->callPlatformFunction (gTBReadFunctionNameSym, false, (void *)&gpioState, 0, 0, 0);
+				//getCPUTimebase (&tbu, &tbl);
+				//kprintf ("4CPU::tb read tb gpio got value %d, current tb values %d, %ud\n", gpioState, tbu, tbl);
+			} while ((gpioState == 0) && (count < 15));
+			
+			//getCPUTimebase (&tbu, &tbl);
+		
+			kprintf ("MacRISC4CPU::enableCPUTimeBase(%s) complete\n", enable ? "enable" : "disable");
+		}
 	} else {
-		kprintf ("MacRISC4CPU::enableCPUTimeBase - I2C read failed\n");
-		return;
+		// Do it messy I2C way 
+		UInt8 sevenBitAddr, buf, tmp;
+		
+		kprintf ("4CPU::tb - timebase sync old way\n");
+	
+		// read the byte register -- requires 7 bit slave address
+		sevenBitAddr = gTimeBaseParams->i2c_addr >> 1;
+		if (kIOReturnSuccess == gI2CDriver->callPlatformFunction (i2c_readI2CBus, false, (void *)(UInt32)sevenBitAddr, (void *) (UInt32)gTimeBaseParams->i2c_subaddr, (void *)(UInt32)&buf, (void *)1)) {
+			// apply mask and value
+			tmp = enable ? gTimeBaseParams->enable_value : gTimeBaseParams->disable_value; 
+			buf = (buf & ~gTimeBaseParams->mask) | (tmp & gTimeBaseParams->mask);
+			gI2CDriver->callPlatformFunction (i2c_writeI2CBus, false, (void *)(UInt32)sevenBitAddr,(void *) (UInt32)gTimeBaseParams->i2c_subaddr, (void *)(UInt32)& buf, (void *)1);
+		} else {
+			kprintf ("MacRISC4CPU::enableCPUTimeBase - I2C read failed\n");
+			return;
+		}
+		/*
+		* We generally get called twice, the first time to disable the timebase and the second
+		* time to enable it.  We're only done after the second call, i.e., enable is true.
+		* Setting gI2CTransactionComplete to true signals startCPU that it is OK to close
+		* the I2C bus.
+		*/
+		gI2CTransactionComplete = enable;
+		kprintf ("MacRISC4CPU::enableCPUTimeBase(%s) - I2C transaction complete\n", enable ? "enable" : "disable");
 	}
-	kprintf ("MacRISC4CPU::enableCPUTimeBase(%s) - I2C transaction complete\n", enable ? "enable" : "disable");
-	/*
-	 * We generally get called twice, the first time to disable the timebase and the second
-	 * time to enable it.  We're only done after the second call, i.e., enable is true.
-	 * Setting gI2CTransactionComplete to true signals startCPU that it is OK to close
-	 * the I2C bus.
-	 */
-	gI2CTransactionComplete = enable;
 
 	return;
 }
